float3 main(
	//The normal to the point being shaded
	//
	//This normal is calculated by the vertex program and stored in TEXCOORD0
	//	It is than interpolated accross all fragments belonging to the triangle.
	//	By binding aNorm to TEXCOORD0, we can read the value for the current fragment.
	in float3 aNorm : TEXCOORD0,
	
	//Textures are passed to fragment programs through samplers
	//	Below we declare aCubeMapTex to be a cubic texture sampler
	//	Textures are always uniform	
	uniform samplerCUBE aCubeMapTex
) : COLOR
{
	//Since the normal is linearly interpolated across the fragments
	//	it's length will be <1 inside the triangle,
	//	so we need to normalize it
	float3 dir = normalize(aNorm);

	//Texture loookups are done with the corresponding texXXXX instrictuion.
	//For cube maps - texCUBE samples the texture specified by the first parameter
	//	using the direction (not position!) specified in the second paramter.
	float3 color = texCUBE(aCubeMapTex, dir);
	
	return color;
}
