void main(
	//The model-view-projection transfomation matrix
	//
	//Uniform parameters stay the same accross all executed vertices
	//	until changed by the main program (on the CPU)
	//
	//This holds for fragment programs as well - uniform parameters
	//	have the same value across all drawn fragments
	uniform float4x4 aMVP,
	
	//The input position of the vertex
	in float4 aPos : POSITION,
	
	//The normal to the vertex
	in float3 aNorm : NORMAL,
	
	//The output (transformed) position of the vertex
	out float4 oPos : POSITION,

	//The untransformed position of the vertex.
	//	This value is used by the fragment program to do texture
	//	lookups in the cube environment map 
	//
	//In general - if you want to pass a parameter from
	//	the vertex program to the fragment program you can do it by 
	//	mapping the paramater to one of the texture coordinates TEXCOORD0-TEXCOORD7.
	//	You would then need to map an input parameter to the same texture coordinate in
	//	the fragment program
	//	Note that texture coordinates are linearly interpolated inside the drawn triangle
	//	before being passed to the fragment program
	out float3 oNorm : TEXCOORD0
)
{
	//Calculate the coordinates of the vertex in camera space coordinates
	oPos = mul(aMVP, aPos);
	
	//Pass the unmodified (almost :) normal to the fragment shader 
	oNorm = normalize(aNorm);
}
