#include "Cg.h"
#include <fstream>

#include <assert.h>
using namespace std;

static CgObjects* g_pCg = NULL;

CgObjects::CgObjects(){
  bool error = false;
  cgNotSupported = false;
  _shaderContext = NULL;
  

  // Setze Error-Callback und erzeuge Profile
  cgSetErrorCallback(cgErrorCallback);
  _shaderContext = cgCreateContext();
  if (_shaderContext == NULL){
	lastErrorString += "CG-Context could not be created\n";
	error = true;	  
  }
  
  // get latest vertex shader profile
  _vertexProfile = cgGLGetLatestProfile (CG_GL_VERTEX);
  
  /*if (cgGLIsProfileSupported(CG_PROFILE_VP20))
        _vertexProfile = CG_PROFILE_VP20;
  else if (cgGLIsProfileSupported(CG_PROFILE_ARBVP1))
        _vertexProfile = CG_PROFILE_ARBVP1;
  else
	  	_vertexProfile = CG_PROFILE_UNKNOWN;
  */
  if (_vertexProfile == CG_PROFILE_UNKNOWN){
		lastErrorString += "No supported Vertex Shader Profile found\n";
		error = true;	  
  }

  // get latest fragment shader profile
  _fragmentProfile = cgGLGetLatestProfile (CG_GL_FRAGMENT);
  
  /*if (cgGLIsProfileSupported(CG_PROFILE_FP30))
        _fragmentProfile = CG_PROFILE_FP30;
  else if (cgGLIsProfileSupported(CG_PROFILE_ARBFP1))
        _fragmentProfile = CG_PROFILE_ARBFP1;
  else
	  	_fragmentProfile = CG_PROFILE_UNKNOWN;
  */
  if (_fragmentProfile == CG_PROFILE_UNKNOWN){
		lastErrorString += "No supported Fragment Shader Profile found\n";
		error = true;	  
  }
    
  // Falls es einen Error gab, so sei der CG nicht unterstützt
  if (error){
  	cgNotSupported = true;
	return;
  }
  
  // aktiviere optimierung
  cgGLSetOptimalOptions (_vertexProfile);
  cgGLSetOptimalOptions (_fragmentProfile);

}

CgObjects::~CgObjects()
{
	if (_shaderContext != NULL)
		cgDestroyContext(_shaderContext);
	
	_shaderContext = NULL;
}

CGprofile CgObjects::fragmentProfile()
{
  return _fragmentProfile;
}
const CGprofile CgObjects::fragmentProfile() const
{
  return _fragmentProfile;
}

CGprofile CgObjects::vertexProfile()
{
  return _vertexProfile;
}
const CGprofile CgObjects::vertexProfile() const
{
  return _vertexProfile;
}

CGcontext CgObjects::shaderContext()
{
  return _shaderContext;
}
const CGcontext CgObjects::shaderContext() const
{
  return _shaderContext;
}

void CgObjects::cgErrorCallback(void)
{
  CGerror LastError = cgGetError();
	
  if(LastError) {
    printf ( "Cg Error.\n" );
    const char *Listing = cgGetLastListing(getCgInstance()->shaderContext());
    printf ( "\n---------------------------------------------------\n");
    printf ("%s\n\n", cgGetErrorString(LastError));
    printf ("%s\n", Listing);
    printf ("---------------------------------------------------\n");
    printf ("Cg error, exiting...\n");
  }
  //assert (false);
}

CgObjects* getCgInstance()
{
  if (g_pCg == NULL) {
    g_pCg = new CgObjects;
  }
  return g_pCg;
}

void releaseCg()
{
  delete g_pCg;
  g_pCg = NULL;
}



/*************************
 *** CgProgram ***
 *************************/

CgProgram::CgProgram()
{
  // programs
  _Program = NULL;
  _Profile = (CGprofile)0xffff;
	
  programIsSource = true;
}

void CgProgram::setFilename( const string &fn )
{
  _filename = fn;
}

void CgProgram::initProfile( CGprofile p )
{
  _Profile = p;
}


CgProgram::~CgProgram()
{
  if (_Program != NULL) {
    cgDestroyProgram(_Program);
	_Program = NULL;
  }
}


/// Setup, loads fragment program from harddrive
/** Also performs setup of global objects, if not already initialized */
int CgProgram::setup()
{
  CgObjects* cgobj = getCgInstance();

  lastErrorString = cgobj->lastErrorString;
	
  if (cgobj->cgNotSupported){
  	this->cgNotSupported = true;
	return 100;	  
  }
  
  if (_Program != NULL) {
    cgDestroyProgram(_Program);
    _Program = NULL;
  }

  // Read programm from our file system
  //int file = 0;
/*  std::ifstream fStream;
  fStream.open(_filename.c_str());
	
  // check if the file could be found
  if (!fStream.good())
  {
        printf("The file \"%s\" was not found", _filename.c_str());
        return 100;
  }

  // get size of the file	
  fStream.seekg(0, std::ios_base::end);
  int size = fStream.tellg();
  fStream.seekg(0, std::ios_base::beg);
	
  // read whole file
  char* pBuf = new char[size];
  memset(pBuf, 0, sizeof(char) * size);
  fStream.read(pBuf, size);

  std::string program = pBuf;

  delete [] pBuf;
  fStream.close();

	// create program	
  	_Program = cgCreateProgram(getCgInstance()->shaderContext(),
				     programIsSource ? CG_SOURCE : CG_OBJECT,
				     program.c_str(),
				     _Profile, NULL, NULL);
*/	
 
  
  _Program = cgCreateProgramFromFile(getCgInstance()->shaderContext(),
				     programIsSource ? CG_SOURCE : CG_OBJECT,
				     _filename.c_str(),
				     _Profile, NULL, NULL);
  
  if (_Program == NULL) {
    lastErrorString += ( "Could not create program %s\n");//, _filename.c_str() );
	return 100;
  }
  cgGLLoadProgram(_Program);
  _parameters.erase( _parameters.begin(), _parameters.end() );
  return 0;
}


/// Enable the fragment program
void CgProgram::enable()
{
  // Use fragment program
  cgGLEnableProfile(_Profile);
  cgGLBindProgram(_Program);
}

/// Disable the fragment program
void CgProgram::disable()
{
  // disable profile
  cgGLDisableProfile( _Profile );
}

/// Read the handle of a parameter of the fragment program
CGparameter CgProgram::parameter( const string &var, int slot )
{
  // First try to find parameter in the map
  map<string, CGparameter>::const_iterator it = _parameters.find( var );
  if (it != _parameters.end() && slot == -1) {
    return (*it).second;
  }
  
  // Obtain parameter handle from Cg
  CGparameter p = cgGetNamedParameter(_Program, var.c_str() );

  if (slot != -1){
	  p = cgGetArrayParameter(p, slot );
  }
  
  if (!p) {
	  char str[256];
	  sprintf (str,"Unable to retrieve program parameter %s, exiting.\n", var.c_str());
    lastErrorString += ( str );
  }
  _parameters[var] = p;
  return p;
}

/// Check if such parameter exists
bool CgProgram::hasParameter(const std::string& pname)
{
  map<string, CGparameter>::const_iterator it = _parameters.find( pname );
  if (it != _parameters.end())
  {
    return true;
  }
  
  CGparameter p = cgGetNamedParameter(_Program, pname.c_str() );
  
  if (!p) return false;
  
  return true;
}


void CgProgram::setParameter( const CGparameter &p, int n, const float *v, bool matrixTranspose  )
{
  switch( n ) {
  case 1:
    cgGLSetParameter1fv( p, v );
    break;
  case 2:
    cgGLSetParameter2fv( p, v );
    break;
  case 3:
    cgGLSetParameter3fv( p, v );
    break;
  case 4:
    cgGLSetParameter4fv( p, v );
    break;
  case 16:
	 if (matrixTranspose)
         cgGLSetMatrixParameterfr(p,v);
	 else
         cgGLSetMatrixParameterfc( p, v );
    break;
  default:
    lastErrorString += ( "Illegal parameter length.\n" );
  }
}

/*void CgProgram::setParameter( const CGparameter &p, int n, const int *v )
{
  switch( n ) {
  case 1:
    cgGLSetParameter1iv( p, v );
    break;
  case 2:
    cgGLSetParameter2iv( p, v );
    break;
  case 3:
    cgGLSetParameter3iv( p, v );
    break;
  case 4:
    cgGLSetParameter4iv( p, v );
    break;
  case 16:
    cgGLSetMatrixParameterir( p, v );
    break;
  default:
    lastErrorString += ( "Illegal parameter length.\n" );
  }
}
void CgProgram::setParameter( const string &pname, int n, const int *v )
{
  setParameter( parameter(pname), n,v );
}
*/

void CgProgram::setParameter( const string &pname, int n, const float *v, bool matrixTranspose  )
{
  setParameter( parameter(pname), n,v, matrixTranspose );
}


void CgProgram::setArrayParameter (const string &pname, int n, const float*v, int arraySlot, bool matrixTranspose )
{
  setParameter( parameter(pname,arraySlot), n,v, matrixTranspose );
}




/*************************
 *** CgVertexProgram ***
 *************************/

CgVertexProgram::CgVertexProgram():CgProgram()
{
}

CgVertexProgram::~CgVertexProgram()
{
}


int CgVertexProgram::setup()
{
  initProfile( getCgInstance()->vertexProfile() );
  return CgProgram::setup();
}



/// Set transformation matrix state
void CgVertexProgram::setStateMatrixParameter( const CGparameter &p,
					       const CGGLenum matrix, const CGGLenum transform ) const
{
  cgGLSetStateMatrixParameter( p, matrix, transform );
}

/// Set transformation matrix state
void CgVertexProgram::setStateMatrixParameter( const string &pname,
					       const CGGLenum matrix, const CGGLenum transform )
{
  setStateMatrixParameter( parameter(pname), matrix, transform );
}




/*************************
 *** CgFragmentProgram ***
 *************************/

CgFragmentProgram::CgFragmentProgram():CgProgram()
{
}

CgFragmentProgram::~CgFragmentProgram()
{
}


/// Setup, loads fragment program from harddrive
/** Also performs setup of global objects, if not already initialized */
int CgFragmentProgram::setup()
{
  initProfile( getCgInstance()->fragmentProfile() );
  int r = CgProgram::setup();
  _textures.erase( _textures.begin(), _textures.end() );  
  return r;
}

/// Enable the fragment program
void CgFragmentProgram::enable()
{
  CgProgram::enable();
  // enable textures
  map<CGparameter,GLuint>::const_iterator it = _textures.begin();
  while (it != _textures.end()) {
    cgGLEnableTextureParameter( (*it).first );
    cgGLSetTextureParameter( (*it).first, (*it).second );
    it++;
  }
}

/// Disable the fragment program
void CgFragmentProgram::disable()
{
  // disable textures
  map<CGparameter,GLuint>::const_iterator it = _textures.begin();
  while (it != _textures.end()) {
    cgGLDisableTextureParameter( (*it).first );
    it++;
  }
  // disable profile
  CgProgram::disable();
}

/// Bind a texture handle to a parameter of the fragment program
void CgFragmentProgram::bindTexture( const string &var, GLuint T )
{
  bindTexture( parameter(var), T );
}

void CgFragmentProgram::bindTexture( const string &var, GLuint T, int arraySlot )
{
  bindTexture( parameter(var, arraySlot), T );
}

/// Bind a texture handle to a parameter of the fragment program
void CgFragmentProgram::bindTexture( const CGparameter &p, GLuint T  )
{
  _textures[p] = T;
  cgGLSetTextureParameter(p, T);
}
