
#include "stdafx.h"

class ProgramError : public std::exception
{
	std::string m_message;
public:
	ProgramError(std::string aMessage) throw()
		: m_message(aMessage)
	{
	}

	virtual const char *what( ) const throw()
	{
		return m_message.c_str();
	}

	virtual ~ProgramError() throw()
	{

	}
};

void __throwError(const char* aMsg, const char* aFile, int aLine)
{
	std::stringstream ssm;
	ssm << aFile << "(" << aLine << ") : " << aMsg << std::endl;

#if defined(_WIN32) && defined(_DEBUG)
	_RPT0(_CRT_WARN, ssm.str().c_str());
#endif

	throw ProgramError(ssm.str());
}

const char* __getGLErrorDescription(GLenum aError)
{
	const static char* errorMessages[] =
	{
		"GL invalid enum",
		"GL invalid value",
		"GL invalid operation",
		"GL stack overflow",
		"GL stack underflow",
		"GL out of memory"
	};

	if(aError == GL_NO_ERROR)
		return "GL no error";
	else if(aError >= 0x500 && aError <= 0x505)
		return errorMessages[aError - 0x500];
	/*
	else if(aError == GL_TABLE_TOO_LARGE)
		return "GL table too large";
		*/
	else
	{
		std::stringstream ssm;
		ssm << "Unknown GL Error 0x" << std::setbase(16) << std::setw(8) << std::setfill('0') << aError;
		return ssm.str().c_str();
	}

}

