// water_sim.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "error.h"
#include "control_and_initialization.h"
#include "variables.h"
#include "Cg.h"

GLuint ghEnvMapTexture;

//Cg Programs
CgFragmentProgram gWaterFragmentProg, gSphereFragmentProg;
CgVertexProgram gWaterVertexProg, gSphereVertexProg;

//-----------------------------------------------------------------------------------------
void drawSky()
{
	//Draw a sphere around the scene and texture it from the cube map

	//Since we are inside the sphere - cull front faces
	glCullFace(GL_FRONT);

	//Bind the vertex and fragment programs used to render the sphere
	gSphereFragmentProg.enable();
	gSphereVertexProg.enable();
	tstGLError();

	//Bind the current model view transformation to the uniform parameter "aMVP" of the veretex shader
	gSphereVertexProg.setStateMatrixParameter("aMVP", CG_GL_MODELVIEW_PROJECTION_MATRIX, CG_GL_MATRIX_IDENTITY);

	//Bind the cube map to the uniform parameter "aCubeMapTex" of the fragment shader
	gSphereFragmentProg.bindTexture("aCubeMapTex", ghEnvMapTexture);

	//Draw the sphere
	glutSolidSphere(SKY_DISTANCE, 64, 64);
	tstGLError();

	//Disable the vertex and fragment programs
	gSphereVertexProg.disable();
	gSphereFragmentProg.disable();
	tstGLError();

	glCullFace(GL_BACK);

}


//-----------------------------------------------------------------------------------------
void drawWater()
{
	// enable water shaders
	gWaterFragmentProg.enable();
	gWaterVertexProg.enable();

	// set parameters
	gWaterVertexProg.setStateMatrixParameter("aMVP", CG_GL_MODELVIEW_PROJECTION_MATRIX, CG_GL_MATRIX_IDENTITY);

	//Draw the water surface
	//	Currently the code below would draw adjacent squares 
	//	of size WATER_GRID_CELL_SIZE x WATER_GRID_CELL_SIZE
	//	that form a recangle specified by it's corner point
	//	(-WATER_EXTENT, -WATER_EXTENT, 0) - (WATER_EXTENT, WATER_EXTENT, 0)
	//
	//To draw the water, you need to fill in the provided vertex program (water_vert.fx).
	//	It should modify the Z coordinate of each vertex appropriatly. You will also
	//	need to calculate the normal to the surface in your vertex program by using
	//	numerical methods
	//
	//Implement reflections in the fragment program by calculating the reflected
	//	eye ray and than using it look up in the envirnoment. Though physically not
	//	correct, the approach gives nice results
	//
	//Keep in mind:
	//	The variable gCurrentFrameNumber (see variables.h) is incremented 1000 / 30 times in a second
	//		and you can use it to drive your wave animation
	//	Also, the environment map texture's handle is contained in ghEnvMapTexture. Have a
	//		look at drawSky() for usage details.
	//	WATER_EXTENT is a constant giving the size of the rendered quad field 
	//	WATER_GRID_CELL_SIZE is the size of a single grid cell
	//
	//It is helpfull to reduce WATER_EXTENT and WATER_GRID_CELL_SIZE while debugging, 
	//	so you can see the whole rectangle
	//You can also show/hide the coordinate system by pressing D

	for(float x = -WATER_EXTENT; x < WATER_EXTENT; x += WATER_GRID_CELL_SIZE)
	{
		glBegin(GL_QUAD_STRIP);

		for(float y = -WATER_EXTENT; y < WATER_EXTENT; y += WATER_GRID_CELL_SIZE)
		{
			glVertex3f(x, y, 0);
			glVertex3f(x + WATER_GRID_CELL_SIZE, y, 0);
		}

		glEnd();
	}

	// disable water shaders
	gWaterVertexProg.disable();
	gWaterFragmentProg.disable();
	tstGLError();
}


//-----------------------------------------------------------------------------------------
void drawCoordSystem()
{
	glDisable(GL_CULL_FACE);
	glDisable(GL_DEPTH_TEST);

	mat4 mvTransf, mvTransfInv;
	glGetFloatv(GL_MODELVIEW_MATRIX, mvTransf.mat_array);
	invert(mvTransfInv, mvTransf);
	vec4 center = mvTransfInv * vec4(0, 0, -1, 1) /* + vec4(0, 2, 0, 0)*/;

	glBegin(GL_LINES);
	glColor3f(1, 0, 0);
	glVertex3f(center.x, center.y, center.z);
	glColor3f(1, 0, 0);
	glVertex3f(center.x + CAMERA_RADIUS / 10., center.y, center.z);

	glColor3f(0, 1, 0);
	glVertex3f(center.x, center.y, center.z);
	glColor3f(0, 1, 0);
	glVertex3f(center.x, center.y + CAMERA_RADIUS / 10., center.z);

	glColor3f(0, 0, 1);
	glVertex3f(center.x, center.y, center.z);
	glColor3f(0, 0, 1);
	glVertex3f(center.x, center.y, center.z + CAMERA_RADIUS / 10.);
	glEnd();

	glEnable(GL_CULL_FACE);
	glEnable(GL_DEPTH_TEST);
}


//-----------------------------------------------------------------------------------------
void displayFunc()
{
	glClearColor(0, 0, 0, 1);
	glClearDepth(1.);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	tstGLError();

	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glEnable(GL_CULL_FACE);

	setupCamera();
	drawSky();
	drawWater();

	if(gbDrawDebugCoordSystem)
		drawCoordSystem();

	glutSwapBuffers();
}


//-----------------------------------------------------------------------------------------
int main(int argc, char* argv[])
{
	try
	{
		intialize(argc, argv);
		glutDisplayFunc(displayFunc);
		glutMainLoop();
		releaseCg();
	}
	catch(std::exception &aEx)
	{
		std::cerr << aEx.what();
	}
	return 0;
}
